/*  dump2bin.c : conversion utility. 
    Copyright (C) 1999  Ludovic LANGE <ludovic.lange@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>

int dump2bin( char * inputName, char * outputName )
{
    FILE * input;
    int output;
    char * outStart = NULL;

    char * inIndex=0;
    int * intIndex;
    struct stat inBuf;
    int status;
    
    unsigned int startaddr;
    unsigned int length;

    unsigned int	addr;				// record starting address
    
    input = fopen( inputName, "r" );
    if( input == NULL )
    {
	perror("dump2bin : input: ");
	exit( errno );
    }
    
//Computes the length of input file
//=================================

    status = stat( inputName, &inBuf );
    
    if( status != 0 )
    {
	fclose( input );
	perror("dump2bin : fstat in: ");
	exit( errno );
    }
    
    printf("dump2bin : input filesize = %d (bytes)\n", (int)inBuf.st_size );
    
//Computes the length of output file
//==================================

    length = (inBuf.st_size / 59) * 16;

    printf("dump2bin: Image length = %X\n", length );

//Zeroes out the output file
//==========================

    output = open( outputName, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR );
    if( output == -1 )
    {
	fclose( input );
	perror("dump2bin : output: ");
	exit( errno );
    }
    
    status = lseek( output, length-1, SEEK_SET );

    if( status  == -1 )
    {
	perror("dump2bin : lseek out: ");
	fclose( input );
	close( output );
	exit( errno );
    }

    write( output, inIndex, 1);

    close( output );

    output = open( outputName, O_RDWR , S_IRUSR | S_IWUSR );
    if( output == -1 )
    {
	perror("dump2bin : output: ");
	exit( errno );
    }
    
//Map the output file in memory
//=============================

    outStart = mmap( (void *)0, (size_t) length, PROT_WRITE , MAP_SHARED ,
	output, (off_t) 0 );

    printf("Output Mem : start=%p length=%X (end=%p)\n",outStart, length, outStart+length);
    if( outStart == MAP_FAILED )
    {
	perror("dump2bin : mmap out: ");
	fclose( input );
	close( output );
	exit( errno );
    }
    
    inIndex = (char *)intIndex;
    
    startaddr = 0;
    
    while( startaddr < length )
    {
	
	
//Transfer the block to output memory
//===================================

	fscanf( input,"%X %X %X %X %X %X %X %X %X %X %X %X %X %X %X %X %X",
	    &addr,
	    outStart + startaddr + 0,
	    outStart + startaddr + 1,
	    outStart + startaddr + 2,
	    outStart + startaddr + 3,
	    outStart + startaddr + 4,
	    outStart + startaddr + 5,
	    outStart + startaddr + 6,
	    outStart + startaddr + 7,
	    outStart + startaddr + 8,
	    outStart + startaddr + 9,
	    outStart + startaddr + 10,
	    outStart + startaddr + 11,
	    outStart + startaddr + 12,
	    outStart + startaddr + 13,
	    outStart + startaddr + 14,
	    outStart + startaddr + 15);
	printf("Reading block : adr=%X\n", addr );

	startaddr += 16;
    
    }
        

//Synchronize the map
//===================
    status = msync( outStart, (size_t) length, MS_ASYNC );

    if( status != 0 )
    {
	perror("dump2bin : msync out: ");
	fclose( input );
	close( output );
	exit( errno );
    }
    
//Synchronize the filesystem
//==========================
    status = fsync( output );

    if( status != 0 )
    {
	perror("dump2bin : fsync out: ");
	fclose( input );
	close( output );
	exit( errno );
    }

//Unmap the filesystem
//====================
    status = munmap( outStart, (size_t) length );
    
    if( status != 0 )
    {
	perror("dump2bin : munmap out: ");
	fclose( input );
	close( output );
	exit( errno );
    }
    
    close( output );
    fclose( input );

    return 0;

}
    
int main( int argc, char * argv[] )
{
    if( argc < 3 )
    {
	printf("dump2bin: <input file> <output file>\n");
	exit( -1 );
    }
        
    dump2bin( argv[1], argv[2] );
    
    return 0;
}
    
    
    
    
    
    
    
    
    