/*  rom2bin.c : conversion utility. 
    Copyright (C) 1999  Ludovic LANGE <ludovic.lange@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>

int rom2bin( char * inputName, char * outputName )
{
    int input;
    int output;
    char * inStart = NULL;
    char * outStart = NULL;

    char * inIndex;
    int * intIndex;
    struct stat inBuf;
    int status;
    
    unsigned int	romStart;
    unsigned int	romLen;
    unsigned int	addr;				
    unsigned int	nbBytes;			// record length
    unsigned int	checksum;			// record checksu,
    unsigned int	computedChecksum;
    unsigned int	index;
    unsigned char	*ptr;
    
    input = open( inputName, O_RDONLY );
    if( input == -1 )
    {
	perror("rom2bin : input: ");
	exit( errno );
    }
    
    
    printf("Rom2Bin : input filedes = %d\n", input );
    
//Computes the size of input file
//===============================

    status = fstat( input, &inBuf );
    
    if( status != 0 )
    {
	perror("rom2bin : fstat in: ");
	close( input );
	exit( errno );
    }
    
    printf("Rom2Bin : input filesize = %d (bytes)\n", (int)inBuf.st_size );
    
//Maps input file in memory
//=========================    
    
    inStart = mmap( (void *)0, (size_t) inBuf.st_size, PROT_READ, MAP_PRIVATE ,
	input, (off_t) 0 );

    printf("Input Mem : start=%p length=%X (end=%p)\n",inStart, (int)inBuf.st_size, inStart+inBuf.st_size);

    if( inStart == MAP_FAILED )
    {
	perror("rom2bin : mmap out: ");
	close( input );
	exit( errno );
    }
    
//Computes the data of rom image
//==============================

    inIndex = inStart + 7;
    
    intIndex = (int *)(inIndex);
    
    romStart = *(intIndex++);
    romLen = *(intIndex++);

    printf("rom2bin: Image start = %X, image length = %X\n", romStart, (size_t)romLen );

//Zeroes out the output file
//==========================

    output = open( outputName, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR );
    if( output == -1 )
    {
	perror("rom2bin : output: ");
	exit( errno );
    }
    
    status = lseek( output, romLen-1, SEEK_SET );

    if( status  == -1 )
    {
	perror("rom2bin : lseek out: ");
	close( input );
	close( output );
	exit( errno );
    }

    write( output, inIndex, 1);

    close( output );

    output = open( outputName, O_RDWR , S_IRUSR | S_IWUSR );
    if( output == -1 )
    {
	perror("rom2bin : output: ");
	exit( errno );
    }
    
//Maps the output file in memory
//==============================

    outStart = mmap( (void *)0, (size_t) romLen, PROT_WRITE , MAP_SHARED ,
	output, (off_t) 0 );

    printf("Output Mem : start=%p length=%X (end=%p)\n",outStart, romLen, outStart+romLen);
    if( outStart == MAP_FAILED )
    {
	perror("rom2bin : mmap out: ");
	close( input );
	close( output );
	exit( errno );
    }
    
    inIndex = (char *)intIndex;
    
    while( 1 )
    {
	
//Computes the data of the block
//==============================

        intIndex = (int *)(inIndex);
	addr = *(intIndex++);
	nbBytes = *(intIndex++);
	checksum = *(intIndex++);	    
        inIndex = (char *)intIndex;

//Computes the CRC
//================
	if( !addr && !checksum)
	{
	    printf("Last one !\n");
	    break;
	}
	else
	{
	    computedChecksum = 0;
	    ptr = (unsigned char *) inIndex;
	    for( index=0; index< nbBytes; index++ )
	    {
	        computedChecksum += *ptr++;
	    }
	}
	
	printf("Reading block : start=%X length=%X checksum read=%X computed=%X\n", addr, nbBytes, checksum, computedChecksum );
	
//Transfers the block to output memory
//====================================

	printf("outStart = %p, outLen = %X, romStart = %X, addr = %X\n", outStart, romLen, romStart, addr);
	printf("Writing at : %p from %p size %X\n",(outStart+addr-romStart), inIndex, nbBytes);
	memcpy( (outStart + addr - romStart), inIndex, nbBytes );    
/*	write( output, inIndex, nbBytes );*/
	inIndex += nbBytes;
    
    }
        

//Unmaps input block
//==================

    status = munmap( inStart, (size_t) inBuf.st_size );
    if( status != 0 )
    {
	perror("rom2bin : munmap in: ");
	close( input );
	close( output );
	exit( errno );
    }

//Synchronize the mapping
//=======================
    status = msync( outStart, (size_t) romLen, MS_ASYNC );

    if( status != 0 )
    {
	perror("rom2bin : msync out: ");
	close( input );
	close( output );
	exit( errno );
    }
    
//Synchronize the filesystem
//==========================
    status = fsync( output );

    if( status != 0 )
    {
	perror("rom2bin : fsync out: ");
	close( input );
	close( output );
	exit( errno );
    }

//Unmaps output file
//==================
    status = munmap( outStart, (size_t) romLen );
    
    if( status != 0 )
    {
	perror("rom2bin : munmap out: ");
	close( input );
	close( output );
	exit( errno );
    }
    
    close( output );
    close( input );
    
    return 0;

}
    
int main( int argc, char * argv[] )
{
    if( argc < 3 )
    {
	printf("rom2bin: <input file> <output file>\n");
	exit( -1 );
    }
        
    rom2bin( argv[1], argv[2] );
    
    return 0;
}
    
    
    
    
    
    
    
    
    