/*  rom2files.c : conversion utility.
    Copyright (C) 1999  Ludovic LANGE <ludovic.lange@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>

int rom2files( char * inputName, char * outputName )
{
    int input;
    int output=0;
    char * inStart = NULL;

    char * inIndex;
    int * intIndex;
    char name[256];
    struct stat inBuf;
    int status;
    unsigned int	romStart;
    unsigned int	romLen;
    unsigned int	addr;
    unsigned int	nbBytes;
    unsigned int	checksum;
    unsigned int	computedChecksum;
    unsigned int	index;
    unsigned char	*ptr;
    
    input = open( inputName, O_RDONLY );
    if( input == -1 )
    {
	perror("rom2files : input: ");
	exit( errno );
    }
    
    
    printf("rom2files : input filedes = %d\n", input );
    
//Computes the size of input file
//===============================

    status = fstat( input, &inBuf );
    
    if( status != 0 )
    {
	perror("rom2files : fstat in: ");
	close( input );
	exit( errno );
    }
    
    printf("rom2files : input filesize = %d (bytes)\n", (int)inBuf.st_size );
    
//Maps input file in memory
//=========================    
    
    inStart = mmap( (void *)0, (size_t) inBuf.st_size, PROT_READ, MAP_PRIVATE ,
	input, (off_t) 0 );

    printf("Input Mem : start=%p length=%X (end=%p)\n",inStart, (int)inBuf.st_size, inStart+inBuf.st_size);

    if( inStart == MAP_FAILED )
    {
	perror("rom2files : mmap out: ");
	close( input );
	exit( errno );
    }
    
//Computes the parameters
//=======================

    inIndex = inStart + 7;
    
    intIndex = (int *)(inIndex);
    
    romStart = *(intIndex++);
    romLen = *(intIndex++);

    printf("rom2files: Image start = %X, image length = %X\n", romStart, (size_t)romLen );

    inIndex = (char *)intIndex;

    while( 1 )
    {
	
//Gets the datas
//==============

        intIndex = (int *)(inIndex);
	addr = *(intIndex++);
	nbBytes = *(intIndex++);
	checksum = *(intIndex++);	    
        inIndex = (char *)intIndex;

//Computes the CRC
//================
	if( !addr && !checksum)
	{
	    printf("Last one !\n");
	    break;
	}
	else
	{
	    printf("Reading block : inIndex=%p start=%X length=%X\n", inIndex, addr, nbBytes );
	    computedChecksum = 0;
	    ptr = (unsigned char *) inIndex;
	    for( index=0; index< nbBytes; index++ )
	    {
	        computedChecksum += *ptr++;
	    }
	    printf("Reading block : checksum read=%X computed=%X\n", checksum, computedChecksum );
	}
	
	
//Opens output file
//=================

	sprintf( name, "%s_%08X.bin", outputName,addr);
	printf("Writing to %s : start %X size %X\n",name, addr, nbBytes);

        output = open( name, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR );
        if( output == -1 )
        {
	    perror("rom2files : output: ");
	    exit( errno );
	}
    
//Transfer data to output file
//============================

	write( output, inIndex, nbBytes );
	inIndex += nbBytes;
    
	close( output );
    
    }
        

//Unmaps input file
//=================

    status = munmap( inStart, (size_t) inBuf.st_size );
    if( status != 0 )
    {
	perror("rom2files : munmap in: ");
	close( input );
	exit( errno );
    }

    close( input );
    
    return 0;

}
    
int main( int argc, char * argv[] )
{
    if( argc < 3 )
    {
	printf("rom2files: <input file> <output file>\n");
	exit( -1 );
    }
        
    rom2files( argv[1], argv[2] );
    
    return 0;
    
}
    
    
